@@ -42,8 +42,8 @@ void ffPrintMonitor(FFMonitorOptions* options)
         {
             uint32_t moduleIndex = result.length == 1 ? 0 : index + 1;
             FF_PARSE_FORMAT_STRING_CHECKED(&key, &options->moduleArgs.key, 2, ((FFformatarg[]){
-                {FF_FORMAT_ARG_TYPE_UINT, &moduleIndex},
-                {FF_FORMAT_ARG_TYPE_STRBUF, &display->name},
+                {FF_FORMAT_ARG_TYPE_UINT, &moduleIndex, "index"},
+                {FF_FORMAT_ARG_TYPE_STRBUF, &display->name, "name"},
             }));
         }
 
@@ -69,16 +69,16 @@ void ffPrintMonitor(FFMonitorOptions* options)
                 buf[0] = '\0';
 
             FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, FF_MONITOR_NUM_FORMAT_ARGS, ((FFformatarg[]) {
-                {FF_FORMAT_ARG_TYPE_STRBUF, &display->name},
-                {FF_FORMAT_ARG_TYPE_UINT, &display->width},
-                {FF_FORMAT_ARG_TYPE_UINT, &display->height},
-                {FF_FORMAT_ARG_TYPE_UINT, &display->physicalWidth},
-                {FF_FORMAT_ARG_TYPE_UINT, &display->physicalHeight},
-                {FF_FORMAT_ARG_TYPE_DOUBLE, &inch},
-                {FF_FORMAT_ARG_TYPE_DOUBLE, &ppi},
-                {FF_FORMAT_ARG_TYPE_UINT16, &display->manufactureYear},
-                {FF_FORMAT_ARG_TYPE_UINT16, &display->manufactureWeek},
-                {FF_FORMAT_ARG_TYPE_STRING, buf},
+                {FF_FORMAT_ARG_TYPE_STRBUF, &display->name, "name"},
+                {FF_FORMAT_ARG_TYPE_UINT, &display->width, "width"},
+                {FF_FORMAT_ARG_TYPE_UINT, &display->height, "height"},
+                {FF_FORMAT_ARG_TYPE_UINT, &display->physicalWidth, "physical-width"},
+                {FF_FORMAT_ARG_TYPE_UINT, &display->physicalHeight, "physical-height"},
+                {FF_FORMAT_ARG_TYPE_DOUBLE, &inch, "inch"},
+                {FF_FORMAT_ARG_TYPE_DOUBLE, &ppi, "ppi"},
+                {FF_FORMAT_ARG_TYPE_UINT16, &display->manufactureYear, "manufacture-year"},
+                {FF_FORMAT_ARG_TYPE_UINT16, &display->manufactureWeek, "manufacture-week"},
+                {FF_FORMAT_ARG_TYPE_STRING, buf, "serial"},
             }));
         }
 
@@ -180,16 +180,16 @@ void ffGenerateMonitorJsonResult(FF_MAYBE_UNUSED FFMonitorOptions* options, yyjs
 void ffPrintMonitorHelpFormat(void)
 {
     FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_MONITOR_MODULE_NAME, "{2}x{3} px - {4}x{5} mm ({6} inches, {7} ppi)", FF_MONITOR_NUM_FORMAT_ARGS, ((const char* []) {
-        "Display name",
-        "Native resolution width in pixels",
-        "Native resolution height in pixels",
-        "Physical width in millimeters",
-        "Physical height in millimeters",
-        "Physical diagonal length in inches",
-        "Pixels per inch (PPI)",
-        "Year of manufacturing",
-        "Nth week of manufacturing in the year",
-        "Serial number",
+        "Display name - name",
+        "Native resolution width in pixels - width",
+        "Native resolution height in pixels - height",
+        "Physical width in millimeters - physical-width",
+        "Physical height in millimeters - physical-height",
+        "Physical diagonal length in inches - inch",
+        "Pixels per inch (PPI) - ppi",
+        "Year of manufacturing - manufacture-year",
+        "Nth week of manufacturing in the year - manufacture-week",
+        "Serial number - serial",
     }));
 }
 
