@@ -4,6 +4,9 @@
 #include "util/stringUtils.h"
 #include "util/mallocHelper.h"
 #include "util/wcwidth.h"
+#include "util/textModifier.h"
+
+#include <locale.h>
 
 static inline uint32_t max(uint32_t a, uint32_t b)
 {
@@ -12,14 +15,26 @@ static inline uint32_t max(uint32_t a, uint32_t b)
 
 static inline uint32_t getWcsWidth(const FFstrbuf* mbstr, wchar_t* wstr, mbstate_t* state)
 {
+    int result = 1;
+    for (uint32_t i = 0; i < mbstr->length; i++)
+    {
+        if (!isascii(mbstr->chars[i]))
+        {
+            result = 0;
+            break;
+        }
+    }
+    if (__builtin_expect(result, 1)) return mbstr->length;
+
     const char* str = mbstr->chars;
     uint32_t wstrLength = (uint32_t) mbsrtowcs(wstr, &str, mbstr->length, state);
-    int result = mk_wcswidth(wstr, wstrLength);
+    result = mk_wcswidth(wstr, wstrLength);
     return result > 0 ? (uint32_t) result : mbstr->length;
 }
 
 void ffPrintSeparator(FFSeparatorOptions* options)
 {
+    setlocale(LC_CTYPE, "");
     mbstate_t state = {};
     bool fqdn = instance.config.modules.title.fqdn;
     const FFPlatform* platform = &instance.state.platform;
@@ -32,9 +47,11 @@ void ffPrintSeparator(FFSeparatorOptions* options)
         + (fqdn ? platform->hostName.length : ffStrbufFirstIndexC(&platform->hostName, '.')); // host name
     ffLogoPrintLine();
 
-    if(options->string.length == 0)
+    if(options->outputColor.length && !instance.config.display.pipe)
+        ffPrintColor(&options->outputColor);
+    if(__builtin_expect(options->string.length == 1, 1))
     {
-        ffPrintCharTimes('-', titleLength);
+        ffPrintCharTimes(options->string.chars[0], titleLength);
     }
     else
     {
@@ -72,7 +89,10 @@ void ffPrintSeparator(FFSeparatorOptions* options)
             }
         }
     }
+    if(options->outputColor.length && !instance.config.display.pipe)
+        fputs(FASTFETCH_TEXT_MODIFIER_RESET, stdout);
     putchar('\n');
+    setlocale(LC_CTYPE, "C");
 }
 
 bool ffParseSeparatorCommandOptions(FFSeparatorOptions* options, const char* key, const char* value)
@@ -86,6 +106,12 @@ bool ffParseSeparatorCommandOptions(FFSeparatorOptions* options, const char* key
         return true;
     }
 
+    if (ffStrEqualsIgnCase(subKey, "output-color"))
+    {
+        ffOptionParseColor(value, &options->outputColor);
+        return true;
+    }
+
     return false;
 }
 
@@ -105,6 +131,12 @@ void ffParseSeparatorJsonObject(FFSeparatorOptions* options, yyjson_val* module)
             continue;
         }
 
+        if (ffStrEndsWithIgnCase(key, "outputColor"))
+        {
+            ffOptionParseColor(yyjson_get_str(val), &options->outputColor);
+            continue;
+        }
+
         ffPrintError(FF_SEPARATOR_MODULE_NAME, 0, NULL, FF_PRINT_TYPE_NO_CUSTOM_KEY, "Unknown JSON key %s", key);
     }
 }
@@ -131,7 +163,7 @@ void ffInitSeparatorOptions(FFSeparatorOptions* options)
         NULL,
         ffGenerateSeparatorJsonConfig
     );
-    ffStrbufInit(&options->string);
+    ffStrbufInitStatic(&options->string, "-");
 }
 
 void ffDestroySeparatorOptions(FFSeparatorOptions* options)
